% floor planning example (with an optimal trade-off curve)
%
% this is an example taken directly from the paper:
%
%   A Tutorial on Geometric Programming (see pages 24-25)
%   by Boyd, Kim, Vandenberghe, and Hassibi.
%
% constants
a = 0.2;
b = 0.5;
c = 1.5;
d = 0.5;

% GP variables
gpvar wa wb wc wd ha hb hc hd

% objective function
obj = max(wa + wb, wc + wd)*(max(ha,hb) + max(hc,hd));

% constraints (now impose the non-changing constraints)
constr = [ ha*wa == a; hb*wb == b; hc*wc == c; hd*wd == d ];

% alpha is the changing parameter
N = 20;
alpha = linspace(1.01,4,N);

min_area = []; status_history = {};
for n = 1:N
  % add constraints that depend on the changing parameter alpha
  constr(5)  = 1/alpha(n) <= ha/wa; constr(6)  = ha/wa <= alpha(n);
  constr(7)  = 1/alpha(n) <= hb/wb; constr(8)  = hb/wb <= alpha(n);
  constr(9)  = 1/alpha(n) <= hc/wc; constr(10) = hc/wc <= alpha(n);
  constr(11) = 1/alpha(n) <= hd/wd; constr(12) = hd/wd <= alpha(n);

  [obj_value, solution, status] = gpsolve(obj, constr);
  min_area(n,1) = obj_value;
  status_history{end+1} = status;
end

% gp_m: something is wrong with the solutions for n = 2, 4, 6, 8
plot(alpha,min_area);
xlabel('alpha'); ylabel('min area');
axis([1 4 2.5 4]);
% print -depsc floor_planning_tradeoff 
