% box volume maximization example (with an optimal trade-off curve)
%
% this is an example taken directly from the paper:
%
%   A Tutorial on Geometric Programming (see pages 8 and 13)
%   by Boyd, Kim, Vandenberghe, and Hassibi.
%
% problem constants
alpha = 0.5; beta = 2; gamma = 0.5; delta = 2;

% varying parameters for an optimal trade-off curve
N = 10;
Aflr = logspace(1,3,N);
Awall = [100 1000 10000];
volume = zeros(length(Awall),N);

% GP variables
gpvar h w d

% objective
obj = h*w*d;

% constraint set (form only constraints that are not changing)
constr = [ alpha <= h/w; h/w <= beta;
           gamma <= d/w; d/w <= delta; ];

% setup various GP problems with varying parameters
for k = 1:length(Awall)
  for n = 1:N
    % add constraints that have varying parameters
    constr(5) = w*d <= Aflr(n);
    constr(6) = 2*h*w + 2*h*d <= Awall(k);

    % solve the GP problem and compute the optimal volume
    [max_volume solution status] = gpsolve(obj, constr, 'max');
    volume(k,n) = max_volume;
  end
end

loglog(Aflr,volume(1,:), Aflr,volume(2,:), Aflr,volume(3,:));
xlabel('Aflr'); ylabel('V');
% print -depsc max_volume_box_tradeoff
