% minimization of Peron-Frobenious norm example
%
% a numerical instance of the example in the book "Convex Optimization"
% by Boyd and Vandenberghe (page 165-167)

% GP variables
gpvar lambda b(4) s(3) v(4) c(2)

% constants
c_nom = [1 1]';
b_nom = [2 3 2 1]';
alpha = [1 1 1 1]'; beta  = [1 1 1 1]';
s_nom = [1 1 3]';
gamma = [1 1 1]'; delta = [1 1 1]';

% objective
obj = lambda;

% constraints
constr = [...
  % inequalities
  b'*v      <= lambda*v(1);
  s(1)*v(1) <= lambda*v(2);
  s(2)*v(2) <= lambda*v(3);
  s(3)*v(3) <= lambda*v(4);
  [0.5; 0.5] <= c; c <= [2; 2];
  % equalities
  b == b_nom.*((ones(4,1)*(c(1)/c_nom(1))).^alpha).*...
              ((ones(4,1)*(c(2)/c_nom(2))).^beta); 
  s == s_nom.*((ones(3,1)*(c(1)/c_nom(1))).^gamma).*...
              ((ones(3,1)*(c(2)/c_nom(2))).^delta);
];

% find the optimal eigenvalue
[opt_lambda solution status] = gpsolve(obj,constr)
assign(solution)

% construct matrix A
A = zeros(4,4);
A(1,:) = b';
A(2,1) = s(1);
A(3,2) = s(2);
A(4,3) = s(3);

eigA = eig(A)

% >> eig(A) (answer checks)
% 
%    0.8041
%   -0.2841
%   -0.0100 + 0.2263i
%   -0.0100 - 0.2263i
